/* ***********************************************************************/
/*                                                                       */
/*                      WiNRADiO G33DDC Excalibur PRO                    */
/*         Ioctl codes and definitions for G33DDC device driver          */
/*                     (C) Copyright 2015 RADIXON.                       */
/*                                                                       */
/* ***********************************************************************/

#ifndef __G33_IOCTL_H__
#define __G33_IOCTL_H__

#define IOCTL_RDX_BASE              0xA00

#define G33_INTERFACE_TYPE_PCIE         0
#define G33_INTERFACE_TYPE_USB          1

#define G33_DDC                     0
#define G33_DDC_COUNT               1

#define RECEIVER_MODEL              2
#define DEVICE_EEPROM_SIZE          8192
#define INTERNAL_DRIVER_VERSION     3
#define DEVICE_STATE_ERROR_HIGH_TEMP    0x00000001

#define G33_IO_STREAM_IN_IF             0
#define G33_IO_STREAM_IN_DDC            1
//#define G33_IO_STREAM_IN_DDC_WIDE       2
//#define G33_IO_STREAM_IN_DDC_PLAYBACK   3
//#define G33_IO_STREAM_OUT_DDC_PLAYBACK  4
//#define G33_IO_STREAM_OUT_DAC           5

#define G33_IO_STREAM_IN_FIRST          G33_IO_STREAM_IN_IF
#define G33_IO_STREAM_IN_LAST           G33_IO_STREAM_IN_DDC
#define G33_IO_STREAM_IN_COUNT          (G33_IO_STREAM_IN_LAST-G33_IO_STREAM_IN_FIRST+1)

//#define G33_IO_STREAM_OUT_FIRST         G33_IO_STREAM_OUT_DDC_PLAYBACK
//#define G33_IO_STREAM_OUT_LAST          G33_IO_STREAM_OUT_DAC
#define G33_IO_STREAM_OUT_COUNT         0	//(G33_IO_STREAM_OUT_LAST-G33_IO_STREAM_OUT_FIRST+1)

#define G33_IO_STREAM_COUNT             2

#define G33_CONFIG_SIZE                 8196
#define G33_MAX_BUFFER_SIZE             6258688

#pragma pack(push,1)

struct g33_usb_control1
{
    uint16_t    params[3];
    void        *buffer;
    uint16_t    size;
    uint16_t    *transferred;
};

struct g33_usb_control2
{
    void        *buffer;
    uint16_t    size;
    uint16_t    *transferred;
};

struct g33_fpga_data
{
    void        *buffer;
    uint32_t    size;
};

struct g33_rf_control {
	uint32_t Atten;
	uint16_t PreselIndexLow;
	uint16_t PreselIndexHigh;
	uint32_t Preamp;
	uint32_t WettingCurrent;
};

struct g33_rf_control2 {
	uint32_t ExternalReference;
	uint32_t Dither;
	uint32_t Spreading;
};

struct g33_ddc1_params
{
    uint16_t    control_word;
    uint16_t    cic1;
    uint16_t    cic2;
    uint16_t    fir_control;
    uint16_t    fir_len;
    uint32_t    tuning_word;
    uint32_t	phase_shift;
};

struct g33_ddc1_start_params
{
    uint32_t                buffer_size;
    uint32_t                bytes_per_sec;
    struct g33_ddc1_params  params;
    uint32_t                *coeffs;
};

struct g33_dac_params
{
    uint32_t    buffer_size;
    uint32_t    bytes_per_sec;
    uint32_t    dac_control;
};

struct g33_state
{
    uint8_t connected:1;
    uint8_t open:1;
    uint8_t power:1;
    uint8_t suspended:1;
    uint8_t temperature:1;
    uint8_t fpga:1;
};

#pragma pack(pop)

#define G33_IOCTL_GET_CONFIG                    _IOR(0x8C,0x00,uint8_t[G33_CONFIG_SIZE])
#define G33_IOCTL_GET_STATE                     _IOR(0x8C,0x01,struct g33_state)
#define G33_IOCTL_SET_POWER                     _IOW(0x8C,0x02,uint8_t)
#define G33_IOCTL_CONFIGURE_FPGA                _IOW(0x8C,0x03,struct g33_fpga_data)

#define G33_IOCTL_RF_CONTROL1                   _IOW(0x8c,0x04,struct g33_rf_control)
#define G33_IOCTL_RF_CONTROL2                   _IOW(0x8c,0x05,struct g33_rf_control2)

#define G33_IOCTL_SET_NOISE_BLANKER             _IOW(0x8C,0x06,uint16_t)
#define G33_IOCTL_IF_START                      _IOW(0x8C,0x07,uint16_t)
#define G33_IOCTL_IF_STOP                       _IO(0x8C,0x08)
#define G33_IOCTL_IF_PUT_BUFFER                 _IOW(0x8C,0x09,void*)
#define G33_IOCTL_IF_GET_BUFFER                 _IOR(0x8C,0x0A,void*)
#define G33_IOCTL_GET_ADC_LEVEL                 _IOR(0x8C,0x0B,uint16_t)
#define G33_IOCTL_DDC1_0_START                  _IOW(0x8C,0x0C,struct g33_ddc1_start_params)
#define G33_IOCTL_DDC1_0_STOP                   _IO(0x8C,0x0D)
#define G33_IOCTL_DDC1_0_SET_FREQUENCY          _IOW(0x8C,0x0E,uint32_t)
#define G33_IOCTL_DDC1_0_PUT_BUFFER             _IOW(0x8C,0x0F,void*)
#define G33_IOCTL_DDC1_0_GET_BUFFER             _IOR(0x8C,0x10,void*)
#define G33_IOCTL_GET_ADC_SAMPLE_RATE           _IOR(0x8C,0x11,uint32_t)

#define G33_IOCTL_IF_CANCEL_BUFFERS             _IO(0x8C,0x50)
#define G33_IOCTL_DDC1_0_CANCEL_BUFFERS         _IO(0x8C,0x51)

#define G33_IOCTL_USB_CONTROL1_READ             _IOR(0x8C,0x60,struct g33_usb_control1)
#define G33_IOCTL_USB_CONTROL1_WRITE            _IOW(0x8C,0x61,struct g33_usb_control1)
#define G33_IOCTL_USB_CONTROL2_READ             _IOR(0x8C,0x62,struct g33_usb_control2)
#define G33_IOCTL_USB_CONTROL2_WRITE            _IOW(0x8C,0x63,struct g33_usb_control2)

#endif // __G33_IOCTL_H__
